;$Author: kenb $
;$Date: 2006-10-11 13:32:51 -0700 (Wed, 11 Oct 2006) $
;$Header: /home/rumba/cdaweb/dev/control/RCS/combined_subtitle.pro,v 1.4 2003/10/29 21:47:01 johnson Exp kovalick $
;$Locker: kovalick $
;$Revision: 8 $
;+------------------------------------------------------------------------
; NAME: Combined_SUBTITLE
; PURPOSE: Take a prepared string for both the title and pi line  of the subtitle 
; and properly place them on the plot.
; CALLING SEQUENCE:
;       project_subtitle, a, pi_list, title
; INPUTS:
;       a = variable structure returned from read_mycdf which
;	will be used to determine the project and pi affiliation (only if
;	the pi_list is too short to be believable).
;	title = string to place at the top of the gif.
;       pi_list - line containing combine PI's to go at the bottom of the plot file
; OUTPUTS:
;
; KEYWORD PARAMETERS:
;
; AUTHOR:
;       Tami Kovalick QSS Group Inc.
;
; MODIFICATION HISTORY: Initial version is a greatly modified version of 
; project_subtitle.
;
;-------------------------------------------------------------------------
PRO combined_subtitle, a, pi_list, title

; write the title (at the top of the page) to the gif file
web_code='CDAWeb'

xyouts,!d.x_size/2,!d.y_size-!d.y_ch_size,title,/DEVICE,ALIGNMENT=0.5,$
charsize=1.5

;check if the PI_List provided ends w/ an "and", if so, remove it.
  pos = strpos(pi_list, 'and', /reverse_search) ;find the 1st
  if (pos ge (strlen(pi_list)-4)) then pi_list = strmid(pi_list,0,pos-1)

; Generate the subtitle

  pi = ' ' & s='' & b = tagindex('PROJECT',tag_names(a))
  if (b(0) ne -1) then begin
   if(n_elements(a.PROJECT) eq 1) then begin 
    pr = break_mystring(a.PROJECT,delimiter='>')
   endif else begin
    pr = break_mystring(a.PROJECT(0),delimiter='>')
   endelse
    if (pr(0) eq 'ISTP') then begin
      s = 'Key Parameter and Survey data (labels K0,K1,K2)'
      s = s + ' are preliminary browse data.' + '!C'
    endif ;ISTP case
    s = s + ' Generated by '+ web_code + ' on ' + systime()
  endif ; Global attribute Project is found

if (strlen(pi_list) lt 5) then begin
    b = tagindex('PI_NAME',tag_names(a))
    if (b(0) ne -1) then begin
     if(n_elements(a.PI_NAME) eq 1) then pi = a.PI_NAME
    endif
    b = tagindex('PI_AFFILIATION',tag_names(a))
    if (b(0) ne -1) then begin
      if((n_elements(a.PI_AFFILIATION) eq 1) and (a.PI_AFFILIATION[0] ne "")) then $
         pi = pi + ' at '+ a.PI_AFFILIATION 
    endif
  
    pi = 'Please acknowledge data provider, ' + pi + ' and '+ web_code +' when using these data.'
endif else pi = pi_list + ' and '+ web_code +' when using these data.'

;Determine the length of the PI line and if too long, split it into two lines

twoline=0L
threeline=0L
  
if (strlen(pi) gt 45) then begin
  ;find the 2nd 'and' in the string
  p = strpos(pi, 'and') ;find the 1st
  ; RCJ 10/29/03  Added the 'else' below or p2 was undefined when p < 0
  if (p gt 0) then p2 = strpos(pi, 'and', p+3) $ ;starting at the 1st 'and', 
	else p2 = p				     ;find the next
  if (p2 gt 0) then begin
    twoline=1L
    pi_line1 = strmid(pi,0,p2+3)
    pi_line2 = strmid(pi,p2+4) ;copies till the end of the string
    if (strlen(pi_line2) gt 60) then begin ; split up this line as well.
	p = strpos(pi_line2, 'and')
	if (p gt 0) then p2 = strpos(pi_line2, 'and', p+3)
	if (p2 gt 0) then begin
	  p3 = strpos(pi_line2, 'and', p2+3)
	  if (p3 gt 0) then begin
	    twoline=0L
  	    threeline = 1L
	    pi_tmp2 = strmid(pi_line2,0,p3+3) ;copies up to the 3rd 'and'
	    pi_line3 = strmid(pi_line2,p3+4) ;copies till the end of the string
	    pi_line2 = pi_tmp2
 	  endif
	endif
    endif
  endif
endif

; Write the subtitle into the window
if (s ne '') then begin
  c = 1.0 ; initialize
  if ((!d.x_ch_size * strlen(s)) gt !d.x_size) then begin
    b = float(!d.x_ch_size * strlen(s)) / float(!d.x_size)
;original    c = 1.0 - (b/7.0)
    c = 1.0 - (b/11.0)
  endif


  if (c ge 0.75) then begin
    if (threeline) then begin
      xyouts,!d.x_size/2,40L,pi_line1,/DEVICE,ALIGNMENT=0.5,CHARSIZE=c
      xyouts,!d.x_size/2,30L,pi_line2,/DEVICE,ALIGNMENT=0.5,CHARSIZE=c
      xyouts,!d.x_size/2,20L,pi_line3,/DEVICE,ALIGNMENT=0.5,CHARSIZE=c
      xyouts,!d.x_size/2,10L,s,/DEVICE,ALIGNMENT=0.5,CHARSIZE=c
    endif else begin
	if (twoline) then begin
          xyouts,!d.x_size/2,30L,pi_line1,/DEVICE,ALIGNMENT=0.5,CHARSIZE=c
          xyouts,!d.x_size/2,20L,pi_line2,/DEVICE,ALIGNMENT=0.5,CHARSIZE=c
          xyouts,!d.x_size/2,10L,s,/DEVICE,ALIGNMENT=0.5,CHARSIZE=c
        endif else begin
          xyouts,!d.x_size/2,20L,pi,/DEVICE,ALIGNMENT=0.5,CHARSIZE=c
          xyouts,!d.x_size/2,10L,s,/DEVICE,ALIGNMENT=0.5,CHARSIZE=c
        endelse
    endelse
  endif 

  if (c lt 0.75) then begin ;TJK added to handle label when
				       ;space is limited
    s = ''
    if (pr(0) eq 'ISTP') then begin
      s = 'Key Parameter and Survey data (labels K0,K1,K2) are '
      s = s + 'preliminary browse data.' + '!C'
    endif

    gen_date = ' Generated by '+ web_code + ' on ' + systime()

    if (threeline) then begin
      xyouts,!d.x_size/2,40L,pi_line1,/DEVICE,ALIGNMENT=0.5,CHARSIZE=c
      xyouts,!d.x_size/2,30L,pi_line2,/DEVICE,ALIGNMENT=0.5,CHARSIZE=c
      xyouts,!d.x_size/2,20L,pi_line3,/DEVICE,ALIGNMENT=0.5,CHARSIZE=c
      xyouts,!d.x_size/2,10L,s,/DEVICE,ALIGNMENT=0.5,CHARSIZE=c
    endif else begin
	if (twoline) then begin
          xyouts,!d.x_size/2,30L,pi_line1,/DEVICE,ALIGNMENT=0.5,CHARSIZE=c
          xyouts,!d.x_size/2,20L,pi_line2,/DEVICE,ALIGNMENT=0.5,CHARSIZE=c
          xyouts,!d.x_size/2,10L,s,/DEVICE,ALIGNMENT=0.5,CHARSIZE=c
        endif else begin
          xyouts,!d.x_size/2,20L,pi,/DEVICE,ALIGNMENT=0.5,CHARSIZE=c
          xyouts,!d.x_size/2,10L,s,/DEVICE,ALIGNMENT=0.5,CHARSIZE=c
        endelse
    endelse

  endif

endif
end
