;+
;NAME:
; Themis_w
;PURPOSE:
; Widget program for THEMIS data Analysis, to get the infomation in
; the widget
; use widget_control, widget_id, uval = state
;  state = {wmaster:master, $    ;master widget id
;           exitbut:exitbut, $   ;exit button id
;           loadbut:loadbut, $   ;load button id
;           plotbut:plotbut, $   ;plot button id
;           histbut:histbut, $   ;history button id
;           sttimebut:sttimebut, $ ;start_time button id
;           entimebut:entimebut, $ ;end_time button id
;           sttime_display:sttime_display, $ ;start_time display id
;           entime_display:entime_display, $ ;end_time display id
;           datalist:datalist, $ ;datalist id
;           plottypelist:plottypelist, $ ;plot type list id
;           dtyplist:dtyplist, $ ;data type list id
;           stationlist:stationlist, $ ;station list id
;           lvllist:lvllist, $   ;data level list id
;           sclist:sclist, $     ;spacecraft list id
;           plot_type:'X', $     ;plot_type
;           st_time:tt0, $       ;start_time, double
;           en_time:tt1, $       ;end_time
;           data:ptr_new(), $    ;a pointer to the data_quants array in the tplot common
;           tplot_vars:ptr_new(), $ ;points to the tplot_vars structure
;           plot_configs:ptr_new(), $ ;points to the plot_configs structure
;           current_config:ptr_new(), $;points to the current_config
;           data_id:ptr_new(), $ ;a pointer to an array of strings, 1 for each loaded dataset
;           active_dset:ptr_new(), $  ;points to the data structures that are 'active'
;           history:ptr_new(init_history), $ ;a string array with the analysis history
;           dtypstring:'mag', $  ;the most recently loaded datatype
;           lvl:'l2', $          ;level 0, 1 or 2
;           sc:'thg', $          ;spacecraft_id, 'thg' is ground magnetometer
;           station:'all'}
;CALLING SEQUENCE:
; themis_w
;HISTORY:
; sep-2006, jmm, jimm@ssl.berkeley.edu
; 13-dec-2006, jmm, changed format of times in history strings, added
; clear_history button
; 14-dec-2006, jmm, added delete_data button
;
;$LastChangedBy J McTiernan$
;$LastChangedDate 14-Dec-2006$
;$LastChangedRevision$
;$URL mailto:jimm@ssl.berkeley.edu $
;
;-
@tplot_com                      ;this is available from the main level
Pro themis_w_event, event

  common themis_w_private, plot_types, data_types, stations, $
    sc_arr, levels, arr_user
  @tplot_com

;Put a catch here to insure that the state remains defined
  err_xxx = 0
  catch, err_xxx
  If(err_xxx Ne 0) Then Begin
    catch, /cancel
    message, /info, 'Error'
    help, /last_message, output = err_msg
    For j = 0, n_elements(err_msg)-1 Do print, err_msg[j]
    If(is_struct(state)) Then Begin
      widget_control, event.top, set_uval = state, /no_copy
    Endif
    Return
  Endif

;  what happened?
  widget_control, event.id, get_uval = uval
  Case uval Of
    'EXIT': widget_control, event.top, /destroy
    'USERLIST':Begin
      widget_control, event.top, get_uval = state, /no_copy
      pindex = widget_info(state.userlist, /list_select)
      npindex = n_elements(pindex)
      For j = 0, npindex-1 Do Begin
        test_str = arr_user[pindex[j]]
        call_procedure, test_str
      Endfor
      thm_ui_update_history, arr_user[pindex], state
      widget_control, event.top, set_uval = state, /no_copy
    End
    'DATALIST':Begin
      widget_control, event.top, get_uval = state, /no_copy
      pindex = widget_info(state.datalist, /list_select)
      pindex = pindex+1
      If(ptr_valid(state.active_dset)) Then ptr_free, state.active_dset
      state.active_dset = ptr_new(pindex)
      tv_names = tnames()
      newnames = tv_names[pindex-1]
      If(ptr_valid(state.active_vnames)) Then ptr_free, state.active_vnames
      state.active_vnames = ptr_new(newnames)
      newnames = ''''+newnames+''''
      npindex = n_elements(pindex)
      If(npindex Eq 1) Then history_ext = ' varnames = '+newnames[0] $
      Else Begin
        history_ext = 'varnames = ['+newnames[0]
        For j = 1, npindex-1 Do $
          history_ext = history_ext+','+newnames[j]
        history_ext = history_ext+']'
      Endelse
      thm_ui_update_history, history_ext, state
      widget_control, event.top, set_uval = state, /no_copy
    End
    'LOAD': Begin
      widget_control, event.top, get_uval = state, /no_copy
      If(is_string(state.dtypstring)) Then dtype = state.dtypstring $
      Else message, 'Choose a data type'
      If(is_string(state.station)) Then station = state.station $
      Else station = 'all'
      spacecraft = *state.sc
      data_level = state.lvl
      tv_names = thm_ui_load_data_fn(state.st_time, state.en_time, $
                                     dtype, station, spacecraft, $
                                     data_level, data_ss)
      If(data_ss[0] Ne -1) Then Begin
        thm_ui_set_state_ptrs, state
        If(ptr_valid(state.active_dset)) Then ptr_free, state.active_dset
        state.active_dset = ptr_new(data_ss)
        If(ptr_valid(state.active_vnames)) Then ptr_free, state.active_vnames
        state.active_vnames = ptr_new(tv_names)
;change the data list display
        display_id = state.datalist
        loaded_ids = *state.data_id
        If(n_elements(loaded_ids) Gt 1) Then loaded_ids = loaded_ids[1:*]
        widget_control, display_id, set_val = loaded_ids
        history_ext = 'varnames = thm_ui_load_data_fn(start_time, end_time, dtype, station, spacecraft, data_level, data_ss)'
        thm_ui_update_history, history_ext, state
      Endif Else Begin
        message, /info, 'Data input is not a valid structure, no loading'
      Endelse
      widget_control, event.top, set_uval = state, /no_copy
    End
    'PTYPE':Begin
      widget_control, event.top, get_uval = state, /no_copy
      pindex = widget_info(state.plottypelist, /list_select)
      state.plot_type = plot_types[pindex] ;in common, this can't not work
      message, /info, 'Plot_type set to; '+plot_types[pindex]
      history_ext = 'plot_type = '''+plot_types[pindex]+''''
      thm_ui_update_history, history_ext, state
      widget_control, event.top, set_uval = state, /no_copy
    End
    'DTYP': begin
      widget_control, event.top, get_uval = state, /no_copy
      pindex = widget_info(state.dtyplist, /list_select)
      state.dtypstring = strmid(data_types[pindex], 0, 3)
;if 'ASI' is chosen, switch the data list
      If(state.dtypstring Eq 'ASI') Then Begin
        stations = thm_ui_asi_stationlist()
        widget_control, state.stationlist, set_val = stations
      Endif Else If(state.dtypstring Eq 'MAG') Then Begin
        stations = thm_ui_asi_stationlist()
        widget_control, state.stationlist, set_val = stations
      Endif
      history_ext = 'dtype = '''+strmid(data_types[pindex], 0, 3)+''''
      thm_ui_update_history, history_ext, state
      widget_control, event.top, set_uval = state, /no_copy
    End
    'LVL': begin
      widget_control, event.top, get_uval = state, /no_copy
      pindex = widget_info(state.lvllist, /list_select)
      state.lvl = levels[pindex]
      history_ext = 'data_level = '''+levels[pindex]+''''
      thm_ui_update_history, history_ext, state
      widget_control, event.top, set_uval = state, /no_copy
    End
    'SC': begin
      widget_control, event.top, get_uval = state, /no_copy
      pindex = widget_info(state.sclist, /list_select)
      If(ptr_valid(state.sc)) Then ptr_free, state.sc
      state.sc = ptr_new(sc_arr[pindex])
      nsc = n_elements(pindex)
      scstr = ''''+sc_arr[pindex]+''''
      If(nsc Eq 1) Then history_ext = 'spacecraft = '+scstr[0] $
      Else Begin
        history_ext = 'spacecraft = ['+scstr[0]
        For j = 1, nsc-1 Do history_ext = history_ext+','+scstr[j]
        history_ext = history_ext+']'
      Endelse
      thm_ui_update_history, history_ext, state
      widget_control, event.top, set_uval = state, /no_copy
    End
    'STATION': begin
      widget_control, event.top, get_uval = state, /no_copy
      pindex = widget_info(state.stationlist, /list_select)
      stp = stations[pindex]
      nstp = n_elements(stp)
      If(nstp Eq 1) Then station_stp = stp Else Begin
        all_chosen = where(stp Eq 'all', nall)
        If(nall Gt 0) Then station_stp = 'all' Else Begin
          station_stp = stp[0]
          For j = 1, nstp-1 Do station_stp = station_stp+' '+stp[j]
        Endelse
      Endelse
      state.station = station_stp
      history_ext = 'station = '''+station_stp+''''
      thm_ui_update_history, history_ext, state
      widget_control, event.top, set_uval = state, /no_copy
    End
    'PLOT': Begin
      widget_control, event.top, get_uval = state, /no_copy
;what is the active dataset?
      data_ss = *state.active_dset
      plot_type = state.plot_type
;uses tplot now, but this will need to become a separate widget
;     tplot, data.varnames
      thm_ui_plot_data, state.st_time, state.en_time, data_ss, $
        state.dtypstring, state.sc, plot_type
      history_ext = 'thm_ui_plot_data, start_time, end_time, data_ss, dtype, $spacecraft, plot_type'
      thm_ui_update_history, history_ext, state
      widget_control, event.top, set_uval = state, /no_copy
    End
    'HIST':Begin
      widget_control, event.top, get_uval = state, /no_copy
      thm_ui_history, state
      widget_control, event.top, set_uval = state, /no_copy
    End
    'CLEAR_HIST':Begin
      widget_control, event.top, get_uval = state, /no_copy
      If(ptr_valid(state.history)) Then Begin
        ptr_free, state.history
        state.history = ptr_new('NONE')
      Endif
      widget_control, event.top, set_uval = state, /no_copy
    End
    'STTIME':Begin
      t0 = thm_ui_tselect_fn()
      If(is_struct(t0)) Then Begin
        t1 = time_double(temporary(t0))
        t0 = time_string(t1)
        widget_control, event.top, get_uval = state, /no_copy
        state.st_time = t1
;Set the value in the text widget too
        display_id = state.sttime_display
        widget_control, display_id, set_val = t0
;set a history value string
        history_ext = 'start_time = '''+time_string(t1)+''''
        thm_ui_update_history, history_ext, state
;set the value in the master widget
        widget_control, event.top, set_uval = state, /no_copy
      Endif Else Begin
        message, /info, 'Bad time selection, No action taken'
      Endelse
    End
    'ENTIME':Begin
      t0 = thm_ui_tselect_fn()
      If(is_struct(t0)) Then Begin
        t1 = time_double(temporary(t0))
        t0 = time_string(t1)
        widget_control, event.top, get_uval = state, /no_copy
        state.en_time = t1
        display_id = state.entime_display
        widget_control, display_id, set_val = t0
        history_ext = 'end_time = '''+time_string(t1)+''''
        thm_ui_update_history, history_ext, state
        widget_control, event.top, set_uval = state, /no_copy
      Endif Else Begin
        message, /info, 'Bad time selection, No action taken'
      Endelse
    End
    'STTIME_DISPLAY':Begin
      widget_control, event.id, get_val = temp_string
      t0 = time_string(temp_string)
      If(is_string(t0)) Then Begin
        t1 = time_double(t0)
        widget_control, event.top, get_uval = state, /no_copy
        state.st_time = t1
        widget_control, event.id, set_val = t0
        history_ext = 'start_time = '''+time_string(t1)+''''
        thm_ui_update_history, history_ext, state
        widget_control, event.top, set_uval = state, /no_copy
      Endif Else Begin
        message, /info, 'Bad time selection, No action taken'
      Endelse
    End
    'ENTIME_DISPLAY':Begin
      widget_control, event.id, get_val = temp_string
      t0 = time_string(temp_string)
      If(is_string(t0)) Then Begin
        t1 = time_double(t0)
        widget_control, event.top, get_uval = state
        state.en_time = t1
        widget_control, event.id, set_val = t0
        history_ext = 'end_time = '''+time_string(t1)+''''
        thm_ui_update_history, history_ext, state
        widget_control, event.top, set_uval = state, /no_copy
      Endif Else Begin
        message, /info, 'Bad time selection, No action taken'
      Endelse
    End
    'CALL_TPLOT':Begin
      thm_ui_call_tplot, event.top
    End
    'DELETE':Begin              ;deletes the active datasets
      widget_control, event.top, get_uval = state, /no_copy
      If(ptr_valid(state.active_vnames)) Then Begin
        vnames = *state.active_vnames
        vnames0 = vnames
        store_data, vnames0, /delete
        thm_ui_set_state_ptrs, state
;you need to reset active dataset
        If(ptr_valid(state.data)) Then Begin
          nnn = n_elements(*state.data)
          If(nnn Gt 1) Then Begin
            data_ss = 1
            tv_names = tnames()
            If(ptr_valid(state.active_dset)) Then ptr_free, state.active_dset
            state.active_dset = ptr_new(data_ss)
            If(ptr_valid(state.active_vnames)) Then ptr_free, state.active_vnames
            state.active_vnames = ptr_new(tv_names[0])
          Endif Else Begin      ;no data left
            If(ptr_valid(state.active_dset)) Then ptr_free, state.active_dset
            If(ptr_valid(state.active_vnames)) Then ptr_free, state.active_vnames
          Endelse
        Endif Else Begin        ;no data left
          If(ptr_valid(state.active_dset)) Then ptr_free, state.active_dset
          If(ptr_valid(state.active_vnames)) Then ptr_free, state.active_vnames
        Endelse
;Update the widget list
        If(ptr_valid(state.data_id[0])) Then Begin
          If(n_elements(*state.data_id) Gt 1) Then Begin
            widget_control, state.datalist, set_val = (*state.data_id)[1:*]
          Endif Else widget_control, state.datalist, set_val = *state.data_id
        Endif
;history is hard...        
        vnames = ''''+vnames+''''
        npindex = n_elements(vnames)
        If(npindex Eq 1) Then history_ext = ' del_varnames = '+vnames[0] $
        Else Begin
          history_ext = 'del_varnames = ['+vnames[0]
          For j = 1, npindex-1 Do $
            history_ext = history_ext+','+vnames[j]
          history_ext = history_ext+']'
        Endelse
        history_ext = [history_ext, 'store_data, /delete, del_varnames']
        thm_ui_update_history, history_ext, state
      Endif Else message, /info, 'No datasets chosen to delete'          
      widget_control, event.top, set_uval = state, /no_copy
    End
  Endcase
  
  Return
End

Pro themis_w, test_dbase_dir = test_dbase_dir

  common themis_w_private, plot_types, data_types, stations, $
    sc_arr, levels, arr_user
  @tplot_com

;for testing
;Call the new themis_init program
  themis_init
  If(keyword_set(test_dbase_dir)) Then Begin
;add a slash if necessary to avoid issues in thm_load_gmag, etc....
    tdd = strcompress(/remove_all, test_dbase_dir)
    sl = strlen(tdd)
    sl1 = strmid(tdd, sl-1, 1)
    If(sl1 Ne '/' And sl1 Ne '\') Then Begin
      os_family = strupcase(!version.os_family)
      If(os_family Eq 'WINDOWS') Then tdd = tdd+'\' Else tdd = tdd+'/'
    Endif
    setenv, 'THEMIS_DATA_DIR='+tdd
    !themis.local_data_dir = getenv('THEMIS_DATA_DIR')
    remote_host_dir = 'http://sprg.ssl.berkeley.edu/data/themis/'
    setenv, 'THEMIS_REMOTE_HOST_DIR'+'='+remote_host_dir
    !themis.remote_data_dir = getenv('THEMIS_REMOTE_HOST_DIR')
  Endif
;get data types, spacecraft and stations
  data_types = thm_ui_datatypes()
  stations = thm_ui_mag_stationlist()
  sc_arr = thm_ui_sclist()
  levels = ['L1', 'L2']

;build master widget
  master = widget_base(/row, title = 'THEMIS data analysis', $
                       /tlb_frame_attr)
  print, 'Master widget id', master
;build controls widget
  controls = widget_base(master, /col, /align_center)
;button widget
  buttons = widget_base(master, /col, /align_center, frame = 5)
;plot type list
  plottype = widget_base(buttons, /col, /align_center, frame = 5)
  plottype_label = widget_label(plottype, value = 'Plot Type')
  plot_types = ['X', 'PNG']
  plottypelist = widget_list(plottype, value = plot_types, $
                             ysize = n_elements(plot_types)+1, uval = 'PTYPE')
;level list
  lvlid = widget_base(buttons, /row, /align_center, /frame)
  flabel = widget_label(lvlid, value = 'Data level')
  lvllist = widget_list(lvlid, value = levels, xsiz = 20, $
                        ysiz = n_elements(levels), uval = 'LVL')
;datatype list
  dtypid = widget_base(buttons, /row, /align_center, /frame)
  flabel = widget_label(dtypid, value = 'Datatype')
  dtyplist = widget_list(dtypid, value = data_types, xsiz = 20, $
                         ysiz = 5, uval = 'DTYP')
;spacecraft list
  scid = widget_base(buttons, /row, /align_center, /frame)
  flabel = widget_label(scid, value = 'Spacecraft ')
  sclist = widget_list(scid, value = sc_arr, xsiz = 20, $
                       ysiz = n_elements(sc_arr), uval = 'SC', $
                       /multiple)
;station list
  stationid = widget_base(buttons, /row, /align_center, /frame)
  flabel = widget_label(stationid, value = 'Station   ')
  stationlist = widget_list(stationid, value = stations, xsiz = 20, $
                            ysiz = 5, uval = 'STATION', $
                            /multiple)
; load button
  load_button = widget_base(buttons, /col, /align_center)
  loadbut = widget_button(load_button, val = ' Load ', uval = 'LOAD', $
                        /align_center)
; plot button
  plot_button = widget_base(buttons, /col, /align_center)
  plotbut = widget_button(plot_button, val = ' Summary Plot ', uval = 'PLOT', $
                        /align_center)
; history button
  hist_button = widget_base(buttons, /col, /align_center)
  histbut = widget_button(hist_button, val = ' History ', uval = 'HIST', $
                        /align_center)
; clear history button
  clear_button = widget_base(buttons, /col, /align_center)
  clearbut = widget_button(clear_button, val = ' Clear History ', uval = 'CLEAR_HIST', $
                        /align_center)
; call tplot button
  tplot_button = widget_base(buttons, /col, /align_center)
  tplotbut = widget_button(tplot_button, val = ' Call Tplot ', uval = 'CALL_TPLOT', $
                        /align_center)

; delete button
  delete_button = widget_base(buttons, /col, /align_center)
  deletebut = widget_button(delete_button, val = ' Delete ', uval = 'DELETE', $
                        /align_center)
; exit button
  exit_button = widget_base(buttons, /col, /align_center)
  exitbut = widget_button(exit_button, val = ' Exit ', uval = 'EXIT', $
                        /align_center)
;time selection widget
  timebase = widget_base(controls, /row, /align_right, frame = 5)
  timeselbase = widget_base(timebase, /column, /align_right, frame = 5)

; sttime button
  sttime_button = widget_base(timeselbase, /col, /align_center)
  sttimebut = widget_button(sttime_button, val = ' Choose Start time ', $
                            uval = 'STTIME', /align_center)
; entime button
  entime_button = widget_base(timeselbase, /col, /align_center)
  entimebut = widget_button(entime_button, val = ' Choose End time ', $
                            uval = 'ENTIME', /align_center)
;time display widget
  timedisplaybase = widget_base(timebase, /column, /align_right, frame = 5)
; sttime text
  sttime_display = widget_text(timedisplaybase, /edit, $
                               uval = 'STTIME_DISPLAY', $
                               val = '2006-08-16/00:00:00', $
                               xsize = 30)
; entime button
  entime_display = widget_text(timedisplaybase, /edit, $
                               uval = 'ENTIME_DISPLAY', $
                               val = '2006-08-17/00:00:00', $
                               xsize = 30)
  tt0 = time_double('2006-08-16/00:00:00')
  tt1 = time_double('2006-08-17/00:00:00')

; The dataset_display widget
  dataset_display = widget_base(controls, /col, /align_center)
  datadisp = widget_base(dataset_display, /col, /align_center)
  datadisplabel = widget_label(datadisp, value = 'Loaded Data')
  datalist = widget_list(datadisp, value = 'No Data loaded', $
                         uval = 'DATALIST', xsiz = 60, ysiz = 20, frame = 5, $
                         /multiple)
; User defined procedures
  If(is_string(getenv('THEMIS_USER_PROC_FILE'))) Then Begin
    user_file = getenv('THEMIS_USER_PROC_FILE')
    arr_user = rd_tfile(user_file)
  Endif Else arr_user = 'None'
  user_display = widget_base(controls, /col, /align_center)
  userdisp = widget_base(user_display, /col, /align_center)
  userdisplabel = widget_label(userdisp, value = 'User Defined Procedures')
  userlist = widget_list(userdisp, value = arr_user, $
                         uval = 'USERLIST', xsiz = 60, ysiz = 10, frame = 5, $
                         /multiple)
;init the history with defaults
  init_history = ['plot_type = ''X''', $
                  'data_level = ''l2''', $
                  'dtype = ''mag''', $
                  'spacecraft = ''thg''', $
                  'station = ''all''', $
                  'start_time = '''+time_string(tt0)+'''', $
                  'end_time = '''+time_string(tt1)+'''']
 
  state = {wmaster:master, $    ;master widget id
           exitbut:exitbut, $   ;exit button id
           loadbut:loadbut, $   ;load button id
           plotbut:plotbut, $   ;plot button id
           histbut:histbut, $   ;history button id
           clearbut:clearbut, $   ;clear_history button id
           tplotbut:tplotbut, $   ;call tplot button id
           deletebut:deletebut, $   ;delete button id
           sttimebut:sttimebut, $ ;start_time button id
           entimebut:entimebut, $ ;end_time button id
           sttime_display:sttime_display, $ ;start_time display id
           entime_display:entime_display, $ ;end_time display id
           datalist:datalist, $ ;datalist id
           userlist:userlist, $ ;userlist id
           plottypelist:plottypelist, $ ;plot type list id
           dtyplist:dtyplist, $ ;data type list id
           stationlist:stationlist, $ ;station list id
           lvllist:lvllist, $   ;data level list id
           sclist:sclist, $     ;spacecraft list id
           plot_type:'X', $     ;plot_type
           st_time:tt0, $       ;start_time, double
           en_time:tt1, $       ;end_time
           data:ptr_new(), $    ;a pointer to the data_quants array in the tplot common
           tplot_vars:ptr_new(), $ ;points to the tplot_vars structure
           plot_configs:ptr_new(), $ ;points to the plot_configs structure
           current_config:ptr_new(), $;points to the current_config
           data_id:ptr_new(), $ ;a pointer to an array of strings, 1 for each loaded dataset
           active_dset:ptr_new(), $  ;points to the data structures that are 'active'
           active_vnames:ptr_new(), $ ;the tplot variable names that are active
           history:ptr_new(init_history), $ ;a string array with the analysis history
           dtypstring:'mag', $  ;the most recently loaded datatype
           lvl:'l2', $          ;level 0, 1 or 2
           sc:ptr_new('thg'), $ ;spacecraft_id, 'thg' is ground magnetometer
           station:'all'}
  widget_control, master, set_uval = state, /no_copy
  widget_control, master, /realize
  xmanager, 'themis_w', master, /no_block

Return
End
