;+
;NAME:
; thm_mk_filename
;PURPOSE:
; creates a themis cdf filename from the input components;
;CALLING SEQUENCE:
; filename=thm_mk_filename(time,sc=sc,level=level,data_type=data_type,$
;                          station=station, version=version)
;INPUT:
; time = file time, can be seconds from 01-01-1970, or a string, 
;        'yyyy-mm-dd/hh:mm:ss'
;OUTPUT:
; filename = e.g.,'thg_l2_mag_bmls_20060313_v01.cdf'
;KEYWORDS:
; sc = spacecraft id , 'tha', 'thb', etc..., default is 'thg' for ground data
; level = level, 'l0','l1','l2', or 0, 1, 2, default is 'l2'
; data_type = data type, 'mag', 'efi', 'fgm', etc..., default is 'mag'
; station = station for ground data, default is 'bmls'
; version = a version number, in a string, such as 'v01', or a
;           number. The default is to insert a '*', as a wild card
;           when searching for files.
; All inputs are scalars, all are case-insensitive, 
;HISTORY:
; 27-nov-2006, jmm, jimm@ssl.berkeley.edu
;
;$LastChangedBy$ J McTiernan
;$LastChangedDate$ 5-Dec-2006
;$LastChangedRevision$ Added comments
;$URL$ mailto:jimm@ssl.berkeley.edu
;
;-
Function thm_mk_filename, time,  sc = sc, level = level, $
                          data_type = data_type, $
                          station = station, version = version, $
                          _extra = _extra
  If(is_string(sc)) Then ss = strcompress(/remove_all, strlowcase(sc[0])) $
  Else ss = 'thg'
  If(n_elements(level) Gt 0) Then Begin
    If(is_string(level)) Then Begin
      ll = strcompress(/remove_all, strlowcase(level[0]))
    Endif Else ll = 'l'+strcompress(/remove_all, string(level[0]))
  Endif Else ll = 'l2'
  If(is_string(data_type)) Then Begin
    dd = strcompress(/remove_all, strlowcase(data_type[0]))
  Endif Else dd = 'mag'
  If(is_string(station)) Then Begin
    st = strcompress(/remove_all, strlowcase(station[0]))
  Endif Else st = 'bmls'
  If(n_elements(version) Gt 0) Then Begin
    If(is_string(version)) Then Begin
      vv = strcompress(/remove_all, strlowcase(version[0]))
    Endif Else Begin
      vvx = strcompress(/remove_all, string(version[0]))
      If(version Lt 10) Then vv = 'v0'+vvx Else vv = 'v'+vvx
    Endelse
  Endif Else vv = '*'

  tt = time_string(time)
; yyyy-mm-dd/hh:mm:ss
  If(is_string(tt)) Then Begin
    tt = strmid(tt, 0, 4)+strmid(tt, 5, 2)+strmid(tt, 8, 2)
  Endif Else Begin
    message, /info, 'Invalid Input Time'
    Return, -1
  Endelse
  If(dd Eq 'mag' And ss Eq 'thg') Then Begin
    Return, ss+'_'+ll+'_'+dd+'_'+st+'_'+tt+'_'+vv+'.cdf'
  Endif Else Return,  ss+'_'+ll+'_'+dd+'_'+tt+'_'+vv+'.cdf'
End


    


