;+
;NAME:
; thm_ui_call_tplot
;PURPOSE:
; A widget interface for calling tplot from the themis_w widget,
; currently of limited usefulness, but expandable
;CALLING SEQUENCE:
; thm_ui_call_tplot, master_widget_id
;INPUT:
; master_widget_id = the id number of the widget that calls this
;OUTPUT:
; none, there are buttons to push for plotting, setting limits, not
; sure what else yet...
;HISTORY:
; 14-dec-2006, jmm, jimm@ssl.berkeley.edu
;
;$LastChangedBy J McTiernan$
;$LastChangedDate 14-Dec-2006$
;$LastChangedRevision$
;$URL mailto:jimm@ssl.berkeley.edu $
;
;-
Pro thm_ui_call_tplot_event, event

  @tplot_com

;Put a catch here to insure that the states remain defined
  err_xxx = 0
  catch, err_xxx
  If(err_xxx Ne 0) Then Begin
    catch, /cancel
    message, /info, 'Error'
    help, /last_message, output = err_msg
    For j = 0, n_elements(err_msg)-1 Do print, err_msg[j]
    If(is_struct(state)) Then Begin
      If(is_struct(wstate)) Then $
        widget_control, state.master_id, set_uval = wstate, /no_copy
      widget_control, event.top, set_uval = state, /no_copy
    Endif
    Return
  Endif

;start here
  widget_control, event.id, get_uval = uval
  Case uval Of
    'EXIT': widget_control, event.top, /destroy
    'PLOT':Begin
;find the tplot variables that are selected, and do the tplot
      widget_control, event.top, get_uval = state, /no_copy
      widget_control, state.master_id, get_uval = wstate, /no_copy
;do the tlimits here...
      t0 = state.st_time &  t1 = state.en_time
      If(is_struct(tplot_vars)) Then Begin
        tlimit, t0, t1
        t0 = ''''+time_string(t0)+''''
        t1 = ''''+time_string(t1)+''''
        history_ext = 'tlimit, '+t0+', '+t1
        thm_ui_update_history, history_ext, wstate
      Endif
      If(ptr_valid(wstate.active_vnames)) Then Begin
        tv_plot = thm_ui_temp_arr2str(*wstate.active_vnames)
      Endif Else Begin
        message, /info, 'No Active Dataset, Plotting it all'
        tv_plot = '*'
      Endelse
      tplot, tv_plot            ;this should do the plotting
      tv_plot = ''''+tv_plot+''''
      history_ext = 'tplot, '+tv_plot
      thm_ui_update_history, history_ext, wstate
      widget_control, state.master_id, set_uval = wstate, /no_copy
      widget_control, event.top, set_uval = state, /no_copy
    End
;avg, takes the given tplot variables, subtracts the mean and creates
;new variables for plotting
    'AVG':Begin
      widget_control, event.top, get_uval = state, /no_copy
      widget_control, state.master_id, get_uval = wstate, /no_copy
      If(ptr_valid(wstate.active_vnames)) Then Begin
        tvn = *wstate.active_vnames
        nnn = n_elements(tvn)
        vn_new = strarr(nnn)
        ds_new = intarr(nnn)
        For j = 0, nnn-1 Do Begin
          vname = tvn[j]
          get_data, vname, data = d
; Subtract off the average
          d.y = d.y-(replicate(1, n_elements(d.x))#average(d.y, 1, /double)) 
; create name and ID
          vn_new[j] = vname+'-d'
; Return data to "TPLOT limbo" with new name
          store_data, vname+'-d', data = temporary(d)
          tplot, /add, vname+'-d'
        Endfor
;change active datasets to new variables
        temp_names = tnames()
        data_ss = where_arr(temp_names, vn_new)
        If(data_ss[0] Ne -1) Then data_ss = data_ss+1
        If(ptr_valid(wstate.active_dset)) Then ptr_free, wstate.active_dset
        wstate.active_dset = ptr_new(data_ss)      
        ptr_free, wstate.active_vnames
        wstate.active_vnames = ptr_new(vn_new)
;Set ptrs and ids in the themis_w state
        thm_ui_set_state_ptrs, wstate
;update the data_id window too
;Update the widget list
        If(ptr_valid(wstate.data_id[0])) Then Begin
          If(n_elements(*wstate.data_id) Gt 1) Then Begin
            widget_control, wstate.datalist, set_val = (*wstate.data_id)[1:*]
          Endif Else widget_control, wstate.datalist, set_val = *wstate.data_id
        Endif
;history of this is not easy...
        history_ext = ['nvars = n_elements(varnames)', $
                       'new_vars = strarr(varnames)', $
                       'For j = 0, nvars -1 Do Begin &$', $
                       '  get_data, varnames[j], data=d &$', $
                       '  d.y = d.y-(replicate(1, n_elements(d.x))#average(d.y, 1, /double)) &$', $
                       '  new_vars[j] = varnames[j]+''-d'' &$', $
                       '  store_data, new_vars[j], data = temporary(d) &$', $
                       '  tplot, /add, new_vars[j]', $
                       'varnames = new_vars']
        thm_ui_update_history, history_ext, wstate
        widget_control, state.master_id, set_uval = wstate, /no_copy
        widget_control, event.top, set_uval = state, /no_copy
      Endif Else message, /info, 'No Active Dataset, Nothing happened'
    End
;the times are handled exactly in the same way as in themis_w
    'STTIME':Begin
      t0 = thm_ui_tselect_fn()
      If(is_struct(t0)) Then Begin
        t1 = time_double(temporary(t0))
        t0 = time_string(t1)
        widget_control, event.top, get_uval = state, /no_copy
        state.st_time = t1
;Set the value in the text widget too
        display_id = state.sttime_display
        widget_control, display_id, set_val = t0
;set the value in the master widget
        widget_control, event.top, set_uval = state, /no_copy
      Endif Else Begin
        message, /info, 'Bad time selection, No action taken'
      Endelse
    End
    'ENTIME':Begin
      t0 = thm_ui_tselect_fn()
      If(is_struct(t0)) Then Begin
        t1 = time_double(temporary(t0))
        t0 = time_string(t1)
        widget_control, event.top, get_uval = state, /no_copy
        state.en_time = t1
        display_id = state.entime_display
        widget_control, display_id, set_val = t0
        widget_control, event.top, set_uval = state, /no_copy
      Endif Else Begin
        message, /info, 'Bad time selection, No action taken'
      Endelse
    End
    'STTIME_DISPLAY':Begin
      widget_control, event.id, get_val = temp_string
      t0 = time_string(temp_string)
      If(is_string(t0)) Then Begin
        t1 = time_double(t0)
        widget_control, event.top, get_uval = state, /no_copy
        state.st_time = t1
        widget_control, event.id, set_val = t0
        widget_control, event.top, set_uval = state, /no_copy
      Endif Else Begin
        message, /info, 'Bad time selection, No action taken'
      Endelse
    End
    'ENTIME_DISPLAY':Begin
      widget_control, event.id, get_val = temp_string
      t0 = time_string(temp_string)
      If(is_string(t0)) Then Begin
        t1 = time_double(t0)
        widget_control, event.top, get_uval = state
        state.en_time = t1
        widget_control, event.id, set_val = t0
        widget_control, event.top, set_uval = state, /no_copy
      Endif Else Begin
        message, /info, 'Bad time selection, No action taken'
      Endelse
    End
  Endcase
  
  Return
End

Pro thm_ui_call_tplot, master_widget_id


;build master widget
  tplot_master = widget_base(/row, title = 'TPLOT data analysis', $
                             /tlb_frame_attr)
;button widget
  buttons = widget_base(tplot_master, /col, /align_center, frame = 5)
;build controls widget
  controls = widget_base(tplot_master, /col, /align_center, frame = 5)


;plot button
  plot_button = widget_base(buttons, /col, /align_center)
  plotbut = widget_button(plot_button, val = ' Plot it', uval = 'PLOT', $
                        /align_center)

;average subtraction button
  avg_button = widget_base(buttons, /col, /align_center)
  avgbut = widget_button(avg_button, val = ' Subtract Avg', uval = 'AVG', $
                        /align_center)

;exit button
  exit_button = widget_base(buttons, /col, /align_center)
  exitbut = widget_button(exit_button, val = ' Exit ', uval = 'EXIT', $
                        /align_center)
;time selection widget
  timebase = widget_base(controls, /row, /align_right, frame = 5)
  timeselbase = widget_base(timebase, /column, /align_right, frame = 5)

;sttime button
  sttime_button = widget_base(timeselbase, /col, /align_center)
  sttimebut = widget_button(sttime_button, val = ' Plot Start time ', $
                            uval = 'STTIME', /align_center)
;entime button
  entime_button = widget_base(timeselbase, /col, /align_center)
  entimebut = widget_button(entime_button, val = ' Plot End time ', $
                            uval = 'ENTIME', /align_center)
;Get the start and end times from the master widget
  widget_control, master_widget_id, get_uval = wstate, /no_copy
  t0 = wstate.st_time & t1 = wstate.en_time
  widget_control, master_widget_id, set_uval = wstate, /no_copy
;time display widget
  timedisplaybase = widget_base(timebase, /column, /align_right, frame = 5)
; sttime text
  sttime_display = widget_text(timedisplaybase, /edit, $
                               uval = 'STTIME_DISPLAY', $
                               val = time_string(t0), $
                               xsize = 30)
; entime button
  entime_display = widget_text(timedisplaybase, /edit, $
                               uval = 'ENTIME_DISPLAY', $
                               val = time_string(t1), $
                               xsize = 30)

; State structure
  state = {tplot_master:tplot_master, $
           exitbut:exitbut, $   ;exit button id
           plotbut:plotbut, $   ;plot button id
           sttimebut:sttimebut, $ ;start_time button id
           entimebut:entimebut, $ ;end_time button id
           sttime_display:sttime_display, $ ;start_time display id
           entime_display:entime_display, $ ;end_time display id
           st_time:t0, $        ;start_time, double
           en_time:t1, $        ;end_time
           master_id:master_widget_id} ;the id of the calling widget

  widget_control, tplot_master, set_uval = state, /no_copy
  widget_control, tplot_master, /realize
  xmanager, 'thm_ui_call_tplot', tplot_master, /no_block

  Return
End
