;+
;NAME:
;thm_ui_load_data_fn
;PURPOSE:
;A widget interface to load CDF data for whatever instrument
;CALLING SEQUENCE:
;varnames = thm_ui_load_data_fn(st_time, En_time, dtype, station, $
;                           id_string,data_ss)
;INPUT:
;st_time, en_time = start and end times in seconds from
;                   1-jan-1970 0:00
;dtype, the type of data, a string, of form 'thg_gmag', for example
;station, the ground station of the gmag data
;spacecraft = the spacecraft_is string, or 'thg' for ground data
;data_level = 'L0','L1', or 'l2'
;OUTPUT:
;varnames = an array of tplot variable names, to pass into tplot
;data_ss = the subscripts of the new data in the tplot data_quants structure
; The tplot common is loaded, or appended
;HISTORY:
; 22-sep-2006, jmm, jimm@ssl.berkeley.edu
; 23-oct-2006, jmm, changed to call cdf2tplot
; 30-oct-2006, jmm, changed again, to call load_thg_mag, also added 
;                   dtype and station as inputs
; 13-nov-2006, jmm, changed the argument list, now the output is a
;                   data id string, added a check for start time > end
;                   time
; 08-dec-2006, krb, changed load_thg_mag to thm_load_gmag
; 13-dec-2006, jmm, Added call to cdf2tplot for data for which there
;                   is no load program...
; 14-dec-2006, jmm, returns an array of variable names and
;                   subscripts, which should be easier to read..
; 15-dec-2006, jmm, Added calls to thm_load_fgm, fit, sst, efi,
;                   ask,spacecraft can be an array
;
;$LastChangedBy J McTiernan$
;$LastChangedDate 15-Dec-2006$
;$LastChangedRevision Changes argument list$
;$URL mailto:jimm@ssl.berkeley.edu $
;
;-
Function thm_ui_load_data_fn, st_time, en_time, $
                              dtype0, station0, $
                              spacecraft0, data_level0, $
                              data_ss, _extra = _extra
  @tplot_com
  otp = -1
  data_ss = -1
;database
  If(getenv('THEMIS_DATA_DIR') Eq '') Then Begin
    themis_init                 ;this should work
  Endif
  dtype = strlowcase(strcompress(dtype0, /remove_all))
  station = strlowcase(strtrim(station0, 2))
  spacecraft = strlowcase(strtrim(spacecraft0, 2))
  data_level = strlowcase(strtrim(data_level0, 2))
;Get the info for the data that has been loaded
  If(is_struct(data_quants)) Then Begin
    tx0 = time_string(data_quants.trange)
    didx0 = data_quants.name+':'+tx0[0, *]+' To '+tx0[1, *]
  Endif Else didx = -1
  If(en_time Le st_time) Then Begin
    message, /info, 'End time is less than Start time?'
    message, /info, 'No data Loaded'
    Return, otp
  Endif
  Case dtype Of
    'mag': Begin
      If(station Eq 'all') Then Begin
        thm_load_gmag, /all, trange = [st_time, en_time]
      Endif Else Begin
        thm_load_gmag, station, trange = [st_time, en_time]
      Endelse
    End
    'asi': Begin
      If(station Eq 'all') Then Begin
        thm_load_ask, /all, trange = [st_time, en_time]
      Endif Else Begin
        thm_load_ask, station, trange = [st_time, en_time]
      Endelse
    End
    'efi':Begin
      scs = strmid(spacecraft, 2, 1)
      thm_load_efi, scs, /all, trange = [st_time, en_time]
    End
    'fgm':Begin
      scs = strmid(spacecraft, 2, 1)
      thm_load_fgm, scs, /all, trange = [st_time, en_time]
    End
    'fit':Begin
      scs = strmid(spacecraft, 2, 1)
      thm_load_fit, scs, /all, trange = [st_time, en_time]
    End
    'sst':Begin
      scs = strmid(spacecraft, 2, 1)
      thm_load_sst, scs, /all, trange = [st_time, en_time]
    End
    Else: Begin              ;use cdf2tplot directly
      thm_ui_check_dbase, st_time, en_time, $
        dtype, station, spacecraft, data_level, files_out
      If(is_string(files_out) Eq 0) Then Begin
        message, /info, 'No files available'
        message, /info, 'No data Loaded'
        Return, otp
      Endif
      cdf2tplot, file = files_out
    End
  Endcase
;What are the new structure elements?
  If(is_struct(data_quants)) Then Begin
    ndq = n_elements(data_quants)
    If(data_ss[0] Eq -1) Then data_ss = 1+lindgen(n_elements(data_quants)-1) $
    Else Begin
      If(is_string(didx0)) Then Begin
        tx = time_string(data_quants.trange)
        didx = data_quants.name+':'+tx[0, *]+' To '+tx[1, *]
        new_flag = bytarr(ndq)
        For j = 0, ndq-1 Do Begin
          fff = where(didx0 Eq didx[j], nfff)
          If(nfff Eq 0) Then new_flag[j] = 1
        Endfor
        data_ss = where(new_flag Eq 1)
      Endif Else data_ss = 1+lindgen(n_elements(data_quants)-1)
    Endelse
;Here restrict to the input time range
    thm_ui_only_trange, st_time, en_time, data_ss
;Here get the tplot variable names
    tplotvars = tnames()
    otp = tplotvars[data_ss-1]
  Endif Else Begin
    message, /info, 'No valid_data_structure'
    data_ss = -1
    otp = -1
  Endelse
  Return, otp
End
