;+
;Procedure: THM_LOAD_GMAG,station
;
;keywords:
;   TRANGE= (Optional) Time range of interest  (2 element array).
;   /VERBOSE : set to output some useful info
;   /SUBTRACT_AVERAGE  ;
;Example:
;   thm_load_gmag,'bmls'
;Notes:
;
; To get an array of valid names make the following call;
;   thm_load_gmag,valid_names=vn
; No further action will be taken.
;
;
;Written by: Davin Larson,   Dec 2006
; $LastChangedBy: davin-win $
; $LastChangedDate: 2006-12-16 01:42:02 -0800 (Sat, 16 Dec 2006) $
; $LastChangedRevision: 118 $
; $URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/trunk/idl/themis/ground/thm_load_gmag.pro $
;-
pro thm_load_gmag,stations,trange=trange,verbose=verbose,all=all $
   ,subtract_average=subavg,downloadonly=downloadonly,valid_names = vstats

themis_init

; Valid station names   (see note above):
vstats='bmls ccnv drby ekat fsim fsmi fykn fyts gako gbay gill hots kapu kian loys mcgr pgeo pina pine ptrs rank rmus swno taho tbd1 tpas ukia whit'
vstats=strsplit(vstats,' ',/extract)
if arg_present(vstats) then return

if keyword_set(all) then stations='*'

stats = strfilter(vstats,stations,delimiter=' ',/string)  ; vstats is the subarray of valid stations

if not keyword_set(stats) then  begin
   print,'Input must be one or more of the following strings:
   print,vstats
   return
endif

if keyword_set(verbose) then printdat,stats,/value,'Stations'


for i=0,n_elements(stats)-1 do begin    ; loop over all stations
  station = stats[i]

  ; Although these file names appear platform dependent, it works well on MS windows! Please don't change
  relpath = 'thg/l2/mag/'+ station + '/'
  prefix = 'thg_l2_mag_' + station + '_'
  ending = '_v01.cdf'
  relpathnames = file_dailynames(relpath,prefix,ending,/YEARDIR,trange=trange)

  ; get files:
  files = file_retrieve(relpathnames,local_root=!themis.local_data_dir,remote_root=!themis.remote_data_dir)

  if keyword_set(verbose) then  print,files

  if  keyword_set(downloadonly) then continue

  cdf2tplot,/all,varnames=vars,file=files,verbose=verbose     ; load data into tplot variables
  varname = 'thg_mag_'+station
  options,/default,varname,ytitle=station+'!cB (nT)',constant=0.

  if keyword_set(subavg) then begin
      get_data,varname,data=d,alim=alim
      if keyword_set(d) then begin
         lng=struct_value(alim,'cdf.vatt.station_longitude',default=!values.f_nan)
         lat=struct_value(alim,'cdf.vatt.station_longitude',default=!values.f_nan)

         ;Note 'lat' and 'lng' could be used to subtract off a model dipole field
         ; help,lng,lat

         d.y -= (replicate(1,n_elements(d.x)) # average(d.y,1,/double) )   ; subtract the average value
         store_data,varname,data=d
      endif
  endif



endfor

end

